"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const cheerio = __importStar(require("cheerio"));
const dateParseRegex = {
    order: {
        regex: /^(\d{1,2})\/(\d{1,2})\/(\d{4})$/,
        replacement: '$3-$2-$1',
    },
    transaction: {
        regex: /^.*?(\d{2})\/(\d{2})\/(\d{4}).*?(\d{2}):(\d{2}).*?$/,
        replacement: '$3-$2-$1 $4:$5',
    },
};
/**
 * Internal generalized parseDate function
 *
 * @internal
 */
exports.parseDate = (dateString, mode) => {
    if (!dateParseRegex[mode]) {
        throw new Error(`Unknown mode '${mode}' in parseDate.`);
    }
    const { regex, replacement } = dateParseRegex[mode];
    const date = new Date(dateString.replace(regex, replacement));
    return Math.floor(date.getTime() / 1000);
};
/**
 * Map month numbers from 1-12 to 0-11 and return -1 for everything else
 *
 * @internal
 */
exports.mapMonthsToZeroBased = (month) => {
    if (typeof month === 'number' && month >= 1 && month <= 12) {
        return month - 1;
    }
    return -1;
};
/**
 * Convert scraped HTML to text and break up into an array if it contains <br>
 *
 * @internal
 */
exports.parseScrapedText = (html) => {
    const result = html.split('<br>');
    result.forEach((value, index) => {
        const $ = cheerio.load('<p>' + value + '</p>');
        result[index] = $('p').text();
    });
    return result.filter(Boolean).join('\n');
};
/**
 * Camelcaseify a given string and remove all non a-z characters
 * @internal
 */
exports.camelCaseify = (inputString, delimiter) => {
    delimiter = delimiter || '_';
    return inputString
        .toLowerCase()
        .replace(new RegExp('[^a-z' + delimiter + ']', 'g'), '')
        .replace(new RegExp('(' + delimiter + '[a-z])', 'g'), a => a.toUpperCase().substr(1));
};
/**
 * Convert currency strings into integers
 * $80.00 will become 8000
 *
 * @internal
 */
exports.dollarToInt = (dollarString) => !dollarString
    ? 0
    : Math.trunc(parseFloat(dollarString.replace('$', '')) * 100);
/**
 * Parse orders date to unix timestamp
 *
 * @internal
 */
exports.parseOrdersDate = (orderDateString) => exports.parseDate(orderDateString, 'order');
/**
 * Parse transaction mode (ferry, bus, train) from the info tables image tag
 *
 * @internal
 */
exports.parseTransactionMode = (transactionMode) => {
    const mode = transactionMode.match(/alt="(.*?)"/);
    return mode ? mode[1] : null;
};
/**
 * Parse transaction date to unix timestamp
 *
 * @internal
 */
exports.parseTransactionDate = (transactionDate) => exports.parseDate(transactionDate, 'transaction');
/**
 * Parse cardinfo array and convert currency strings to numbers
 *
 * @internal
 */
exports.parseCardInfo = (cardDetailsString) => {
    let cards;
    try {
        cards = JSON.parse(cardDetailsString);
    }
    catch (e) {
        throw e;
    }
    return cards.map((element, index) => {
        const intermediateCard = Object.assign({}, element);
        intermediateCard.cardIndex = index;
        intermediateCard.cardBalance = exports.dollarToInt(element.cardBalanceInDollars);
        intermediateCard.currentCardBalance = exports.dollarToInt(element.currentCardBalanceInDollars);
        intermediateCard.svPending = exports.dollarToInt(element.svPendingInDollars);
        delete intermediateCard.cardBalanceInDollars;
        delete intermediateCard.currentCardBalanceInDollars;
        delete intermediateCard.svPendingInDollars;
        return intermediateCard;
    });
};
/**
 * Remove hyphens from summary string
 *
 * @internal
 */
exports.sanitizeSummaryString = (summary) => summary.replace(/&#xAD;/g, '');
/**
 * Parse card transaction HTML table to return an data object
 *
 * @internal
 */
exports.parseTransactions = (html) => {
    const $ = cheerio.load(html);
    const data = [];
    const noData = $('.no-activity-list').text();
    if (noData !== '') {
        return [];
    }
    $('#transaction-data tbody tr').each((rowIndex, tableRow) => {
        const cells = [];
        $(tableRow)
            .find('td')
            .each((cellIndex, tableCell) => {
            cells.push($(tableCell).html() || '');
        });
        const [transaction, date, mode, details, journey, fareApplied, fare, discount, amount,] = cells;
        const dataJson = {
            transactionNumber: parseInt(transaction, 10),
            timestamp: exports.parseTransactionDate(date),
            summary: exports.sanitizeSummaryString(details) || null,
            mode: exports.parseTransactionMode(mode) || null,
            fare: {
                applied: fareApplied || null,
                price: Math.abs(exports.dollarToInt(fare)) || 0,
                discount: Math.abs(exports.dollarToInt(discount)) || 0,
                paid: Math.abs(exports.dollarToInt(amount)) || 0,
            },
            journey: null,
        };
        if (dataJson.mode && /^(ferry|bus|train)$/.test(dataJson.mode)) {
            const journeyData = dataJson.summary.split(' to ');
            if (journeyData.length === 2) {
                dataJson.journey = {
                    number: parseInt(journey, 10),
                    start: journeyData[0],
                    end: journeyData[1],
                };
            }
        }
        data.push(dataJson);
    });
    return data;
};
/**
 * ParseAccount description
 *
 * @internal
 */
exports.parseAccount = (html) => {
    const $ = cheerio.load(html);
    const result = {};
    $('#content #tab-5 .column tbody tr').each((rowIndex, tableRow) => {
        const cells = [];
        $(tableRow)
            .find('td')
            .each((cellIndex, tableCell) => {
            cells.push($(tableCell).html() || '');
        });
        result[exports.camelCaseify(cells[0], ' ')] = exports.parseScrapedText(cells[1]);
    });
    return {
        firstName: result.firstName,
        lastName: result.lastName,
        address: result.address,
        dateOfBirth: result.dateOfBirth,
        phoneNumber: result.phoneNumber,
        mobileNumber: result.mobileNumber,
        emailAddress: result.emailAddress,
        nameOnCard: result.nameOnCard,
        cardType: result.cardType,
        cardNumber: result.cardNumber,
        cardExpires: result.cardExpires,
        password: result.password,
        opalPin: result.opalPin,
        securityQuestion: result.securityQuestion,
        securityAnswer: result.securityAnswer,
    };
};
/**
 * ParseOrders description
 *
 * @internal
 */
exports.parseOrders = (html) => {
    const $ = cheerio.load(html);
    const data = [];
    $('#content #tab-5 #transaction-data tbody tr').each((rowIndex, tableRow) => {
        const cells = [];
        $(tableRow)
            .find('td')
            .each((cellIndex, tableCell) => {
            cells.push($(tableCell).html() || '');
        });
        data.push({
            orderId: cells[0],
            orderDate: new Date(exports.parseOrdersDate(cells[1]) * 1000).toISOString(),
            orderDateTimestamp: exports.parseOrdersDate(cells[1]),
            cardType: cells[2],
            orderStatus: cells[3],
        });
    });
    return data;
};
/**
 * Check if pathname starts with /login to determinate if the url request
 * redirected to the login page and thus needs authorization
 *
 * @internal
 */
exports.authorizationNeeded = (pathname) => {
    if (/^\/login/.test(pathname)) {
        return true;
    }
    return false;
};
